//---------------------------------------------------------------------------
// WindowSnap.cpp
//
// Version: 1.0.1
// Author: Moochos
//---------------------------------------------------------------------------
#include <windows.h>
#pragma hdrstop
#include "WindowSnap.h"
#include "WheelCommon.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace WheelHere { namespace Wheeler { namespace WindowSnap {
//---------------------------------------------------------------------------
void TWindowSnap::Update(int AScreenX, int AScreenY)
{
	UpdateTargetWindow(AScreenX, AScreenY);
	UpdateTopLevelWindow();
	UpdateOwnerWindow();
}
//---------------------------------------------------------------------------
void TWindowSnap::UpdateTargetWindow(int AScreenX, int AScreenY)
{
	FTargetWindow = GetWindowAt(AScreenX, AScreenY);
	if(TargetWindow == NULL) {
		throw EWindowError();
	}
	{
		LONG_PTR style = EnsureGetWindowLongPtr(TargetWindow, GWL_STYLE);
		if((style & WS_DISABLED) != 0) {
			FEnabled = false;
		}
		if((style & WS_VISIBLE) == 0) {
			FVisible = false;
		}
	}
	{
		static const LRESULT focusMask =
			DLGC_BUTTON |
			DLGC_DEFPUSHBUTTON |
			DLGC_HASSETSEL |
			DLGC_RADIOBUTTON |
			// Skip: DLGC_STATIC |
			DLGC_UNDEFPUSHBUTTON |
			DLGC_WANTALLKEYS |
			DLGC_WANTARROWS |
			DLGC_WANTCHARS |
			DLGC_WANTMESSAGE |
			DLGC_WANTTAB;
		FWantsFocus = (::SendMessage(TargetWindow, WM_GETDLGCODE, 0, 0) & focusMask) != 0;
	}
}
//---------------------------------------------------------------------------
void TWindowSnap::UpdateTopLevelWindow()
{
	HWND curWindow = TargetWindow;
	while(curWindow != NULL) {
		LONG_PTR style = EnsureGetWindowLongPtr(curWindow, GWL_STYLE);
		if((style & WS_DISABLED) != 0) {
			FEnabled = false;
		}
		if((style & WS_VISIBLE) == 0) {
			FVisible = false;
		}
		if((style & WS_CHILD) == 0) {
			FTopLevelWindow = curWindow;
			return;
		}
		if(MdiChildWindow == NULL &&
			(EnsureGetWindowLongPtr(curWindow, GWL_EXSTYLE) & WS_EX_MDICHILD) != 0) {
			FMdiChildWindow = curWindow;
		}
		curWindow = EnsureGetParent(curWindow);
	}
}
//---------------------------------------------------------------------------
void TWindowSnap::UpdateOwnerWindow()
{
	HWND curWindow = TopLevelWindow;
	while(curWindow != NULL) {
		HWND ownerWindow = EnsureGetWindow(curWindow, GW_OWNER);
		if(ownerWindow == NULL) {
			FOwnerWindow = curWindow;
			if((EnsureGetWindowLongPtr(curWindow, GWL_STYLE) & WS_DISABLED) != 0) {
				HWND popupWindow = ::GetLastActivePopup(curWindow);
				if(popupWindow != curWindow) {
					FModalWindow = popupWindow;
				}
			}
			return;
		}
		curWindow = ownerWindow;
	}
}
//---------------------------------------------------------------------------
HWND TWindowSnap::GetWindowAt(int AScreenX, int AScreenY)
{
	HWND LParentWindow = ::WindowFromPoint(MakePoint(AScreenX, AScreenY));
	if(LParentWindow == NULL) {
		throw EWindowError();
	}
	POINT LClientPoint = MakePoint(AScreenX, AScreenY);
	if(!::ScreenToClient(LParentWindow, &LClientPoint)) {
		throw EWindowError();
	}
	HWND LChildWindow = ::ChildWindowFromPointEx(LParentWindow, LClientPoint, CWP_ALL);
	if(LChildWindow == NULL) {
		return LParentWindow;
	}
	return LChildWindow;
}
//---------------------------------------------------------------------------
TWindowSnap::TWindowSnap(int AScreenX, int AScreenY) :
	FTargetWindow(NULL),
	FMdiChildWindow(NULL),
	FTopLevelWindow(NULL),
	FOwnerWindow(NULL),
	FModalWindow(NULL),
	FEnabled(true),
	FVisible(true),
	FWantsFocus(false)
{
	Update(AScreenX, AScreenY);
}
//---------------------------------------------------------------------------
TWindowSnap::TWindowSnap() :
	FTargetWindow(NULL),
	FMdiChildWindow(NULL),
	FTopLevelWindow(NULL),
	FOwnerWindow(NULL),
	FModalWindow(NULL),
	FEnabled(true),
	FVisible(true),
	FWantsFocus(false)
{
	// Do nothing
}
//---------------------------------------------------------------------------
TWindowSnap::~TWindowSnap()
{
	// Do nothing
}
//---------------------------------------------------------------------------
}}}} // End of namespace Moochos::WheelHere::Wheeler::WindowSnap
//---------------------------------------------------------------------------

